/**
 * Header file for vector_file_reader (vector file reader).
 *
 * Loosely based on the work of Nathan Cassee (TU/e).
 *
 * Part of Master Project 'Porting tree-based hash table compression to GPGPU model checking', 
 * University of Twente (UT).
 *
 * @author  Danny Bergsma (s0113360)
 * @version 1.0
 */

#ifndef VECTOR_FILE_READER_H
#define VECTOR_FILE_READER_H


	// -- INCLUDES --


#include <cstddef>
#include <cstdint>


	// -- TYPEDEFS --


/** Type used for vector elements. */
typedef uint32_t element_t;


	// -- STRUCTS --


/**
 * Struct that represents a collection of vectors; all vectors have the same number of elements.
 */
struct Vectors {
	/**
	 * Number of vectors.
	 * @invariant (nVectors > 0) ==> (vectorLength > 0)
	 * @invariant (nVectors > 0) <==> (elements != NULL)
	 */
	size_t       nVectors     = 0;
	/**
	 * Number of elements in each vector.
	 * @invariant (nVectors > 0) ==> (vectorLength > 0)
	 */
	unsigned int vectorLength = 0;
	/**
	 * The elements of all vectors as one big array.
	 * @invariant (elements != NULL) ==> (length of elements array == (nVectors * vectorLength))  
	 */
	element_t    *elements    = NULL;
};


	// -- FUNCTION DECLARATIONS --


/**
 * Reads vector file with file name fileName. The contents of the file, i.e., the vectors, are returned.
 * Each line of the file should contain one vector, with its elements separated by commas.
 * @param  fileName file name of the vector file
 * @return contents of the file, i.e., the vectors
 */
Vectors readVectorFile(/** @nullable */ const char *fileName);

#endif  // !VECTOR_FILE_READER_H